﻿jQuery(function($) {
	"use strict";

	Proj4js.defs["EPSG:28992"] = "+proj=sterea +lat_0=52.15616055555555 +lon_0=5.38763888888889 +k=0.9999079 +x_0=155000 +y_0=463000 +ellps=bessel +units=m +no_defs";

	var map;
	var markers;
	var geolocate;
	//var lon = 4.9;
	var lon = 121850.0;
	//var lat = 52.366667;
	var lat = 487000.0;
	var zoom = 12;

	/*
	var bounds = new OpenLayers.Bounds(
		12783.622999195053, 305000.0000019111,
		282500.0000089842, 620000.0000000037
	);
	*/

	var bounds = new OpenLayers.Bounds(
		 12700.0, 300000.0,
		282700.0, 620200.0
	);

	//var landscapeHeight;
	var toggleArrowDown = true;

	initTimestamps();

	$(window).load(function() {
		initMap();
		activateLastTimestamp();

		$("#showAbout").fastClick(function() {
			showAboutInfo();
		});

		$("#closeAbout").fastClick(function() {
			closeAboutInfo();
		});

		$("#showMapLegend").fastClick(function() {
			showMapLegend();
		});

		$(".ui-icon-location").fastClick(function() {
			//browserGeoLoocate();
			locateUser();
		});

		$("#sendScreenshotByEmail").fastClick(function() {
			sendScreenshotByEmail();
		});

		$("#toggleTimestamp").fastClick(function() {
			toggleTimestamps();
		});

		//getInitialViewporterSizes();

		window.onorientationchange = orientationChanged;

		setMapLegend();
	});

	$(window).bind('viewportchange', function() {
		updateMapSize();
		reloadTimestamps();
	});

	/*
	function getInitialViewporterSizes() {
		var initialWidth = $('#viewporter').width();
		var initialHeight = $('#viewporter').height();

		landscapeHeight = (initialWidth < initialHeight) ? initialWidth : initialHeight;
	}

	function setViewporterHeight() {
		$('#viewporter').height(landscapeHeight);
	}
	*/

	function toggleTimestamps() {
		$("#timestamp").slideToggle('swing');
		var button = $("#toggleTimestamp");
		if (toggleArrowDown) {
			button.removeClass('ui-icon-arrow-d');
			button.addClass('ui-icon-arrow-u');
		} else {
			button.removeClass('ui-icon-arrow-u');
			button.addClass('ui-icon-arrow-d');
			reloadTimestamps();
		}
		toggleArrowDown = !toggleArrowDown;
	}

	function showAboutInfo() {
		$.mobile.pageContainer.pagecontainer('change', '#aboutDialog', {
			transition: 'flip',
			changeHash: true,
			positionTo: 'window',
			role: 'dialog'
		});
	}

	function closeAboutInfo() {
		$('#aboutDialog').dialog('close');
		viewporter.refresh();
	}

	function showMapLegend() {
		$("#popupLegend").popup("open", {
			positionTo: "window"
		});
	}

	function closeMapLegend() {
		$('#popupLegend').popup('close');
	}

	function setMapLegend() {
		var legendImage = viewporter.isLandscape()
			? "images/legend-landscape.png"
			: "images/legend-portrait.png";

		$('#legendImage').removeAttr("src");
		$('#legendImage').attr("src", legendImage);
	}

	function orientationChanged() {
		switch (window.orientation) {
		case 0:
		case 180:
			// portrait mode
			break;
		case 90:
		case -90:
			// landscape mode
			//setViewporterHeight();
			break;
		}
		//reloadTimestamps();

		setMapLegend();
		setTimeout(function() {
			if (legendIsOpened) {
				$("#popupLegend").popup("reposition", { positionTo: "window" });
			}
		}, 1000);
	}

	function initMap() {
		var base_layer = new OpenLayers.Layer.WMS("HillShade",
			config.url,
			{
				layers: config.baseLayer.name,
				transparent: config.baseLayer.isTransparent,
			},
			{ isBaseLayer: true }
		);

		var options = {
			controls: [],
			maxExtent: bounds,
			restrictedExtent: bounds,
			//maxResolution: 1230.4687499925494,
			numZoomLevels: config.numZoomLevels,
			projection: "EPSG:28992",
			displayProjection: new OpenLayers.Projection("EPSG:4326"),
			units: 'm'
		};

		map = new OpenLayers.Map('map', options);
		map.addLayer(base_layer);

		$.each(config.layers, function(index, layer) {
			map.addLayer(new OpenLayers.Layer.WMS(
				layer.title,
				config.url,
				{
					layers: layer.name,
					transparent: config.isTransparent,
					//bgcolor: '0x000000'
				},
				{
					opacity: config.opacity,
					isBaseLayer: false,
					visibility: false,
				}
			));
		});

		geolocate = createGeolocateControl();

		// Add LayerSwitcher control
		//map.addControl(new OpenLayers.Control.Geolocate());
		//map.addControl(new OpenLayers.Control.LayerSwitcher());
		//map.addControl(new OpenLayers.Control.MousePosition());
		map.addControls([
			//new OpenLayers.Control.LayerSwitcher(),
			//new OpenLayers.Control.MousePosition(),
			//new OpenLayers.Control.NavToolbar(),
			new OpenLayers.Control.Navigation(),
			//new OpenLayers.Control.Scale(),
			//new OpenLayers.Control.ScaleLine(),
			//new OpenLayers.Control.Zoom(),
			//new OpenLayers.Control.ZoomPanel(),
			//new OpenLayers.Control.Geolocate()
			geolocate
		]);

		// Create and add to the map, the marker layer, where the marker will be placed
		markers = new OpenLayers.Layer.Markers("Markers");
		map.addLayer(markers);

		// Set view to zoom maximum map extent
		map.zoomToMaxExtent();
		//map.setCenter(new OpenLayers.LonLat(lon, lat), zoom);

		//layerSwitcher.maximizeControl();
	}

	function updateMapSize() {
		setTimeout(function() {
			map.updateSize();
		}, 1000);
	}

	function initTimestamps() {
		var list = $("#centered ul");
		$.each(config.layers, function(index, layer) {
			var listItem = $("<li>")
				.attr("data-layer-name", layer.name)
				.attr("title", layer.title)
				.attr("onclick", "")
				.click(function() { switchToLayer(layer.title); })
				.append(layer.title);

			list.append(listItem);
		});
	}

	function reloadTimestamps() {
		var $frame = $('#centered');
		$frame.sly('reload');
	}

	function activateLastTimestamp() {
		//var lastLayerName = $("#centered ul li:last-child").attr("title");
		var lastLayerName = $("#centered ul li:nth-last-child(2)").attr("title");
		console.info("LayerName: %s", lastLayerName);
		switchToLayer(lastLayerName);
	}

	function switchToLayer(layerName) {
		hideAllOverlayLayers();
		var layer = map.getLayersByName(layerName)[0];
		layer.setVisibility(true);
		window.currentLayerName = layerName;
	}

	function hideAllOverlayLayers() {
		$.each(map.layers, function(index, layer) {
			if (!layer.isBaseLayer && layer.name != "Markers")
				layer.setVisibility(false);
		});
	}

	function showProgressIndicator() {
		$.mobile.loading( "show", {
			text: "Please wait...",
			textVisible: true,
			textonly: false,
			//theme: $.mobile.loader.prototype.options.theme,
			//html: ""
		});
	}

	function hideProgressIndicator() {
		$.mobile.loading("hide");
	}

	function printObject(obj) {
		if (obj == null)
			return "???";

		var result = obj.toString() + ":\n";
		var propertyValue;
		for (var propertyName in obj) {
			propertyValue = obj[propertyName];
			result += propertyName + " : " + propertyValue + "\n";
		}
		return result;
	}

	function getPositionErrorCode(positionError) {
		switch (positionError.code) {
		case positionError.PERMISSION_DENIED:
			return "PERMISSION_DENIED";
		case positionError.POSITION_UNAVAILABLE:
			return "POSITION_UNAVAILABLE";
		case positionError.TIMEOUT:
			return "TIMEOUT";
		default:
			return "UNKNOWN_ERROR";
		}
	}

	function getPositionErrorMessage(positionError) {
		var positionErrorCode = getPositionErrorCode(positionError);
		var message =
				"code: " + positionErrorCode + "\n" +
				"message: " + positionError.message + "\n";
		return message;
	}

	function getPositionMessage(position) {
		var message =
				"Latitude: " + position.coords.latitude + "\n" +
				"Longitude: " + position.coords.longitude + "\n" +
				"Altitude: " + position.coords.altitude + "\n" +
				"Accuracy: " + position.coords.accuracy + "\n" +
				"Altitude Accuracy: " + position.coords.altitudeAccuracy + "\n" +
				"Heading: " + position.coords.heading + "\n" +
				"Speed: " + position.coords.speed + "\n" +
				"Timestamp: " + position.timestamp + "\n";
		return message;
	}

	function createGeolocateControl() {
		return new OpenLayers.Control.Geolocate({
			eventListeners: {
				"locationupdated": function(event) {
					hideProgressIndicator();
					//alert("Location Updated");
					//alert(printObject(event));

					var position = event.position;
					var message = getPositionMessage(position);
					console.log(message);
					//alert(message);

					var point = event.point;
					locateMarker(point);
				},
				"locationfailed": function(event) {
					hideProgressIndicator();
					//alert("Location Failed");
					//alert(printObject(event));

					var positionError = event.error;
					var message;
					message = getPositionErrorMessage(positionError);
					//console.log("positionError.code: " + positionError.code); //TODO
					console.log(message);
					//alert(message);

					message = "Location detection failed";
					console.log(message);
					alert(message);
				},
				"locationuncapable": function() {
					hideProgressIndicator();
					alert("Location Uncapable");
				}
			},
			geolocationOptions: { enableHighAccuracy: true, timeout: 10000, maximumAge: 5000 },
		});
	}

	function locateMarker(point) {
		// Remove any existing marker
		markers.clearMarkers();

		// Create the icon to be used by the marker
		var size = new OpenLayers.Size(25, 41);
		var offset = new OpenLayers.Pixel(-(size.w / 2), -size.h);
		var icon = new OpenLayers.Icon("images/marker-icon.png", size, offset);
		//icon.setOpacity(0.7);

		// Create a lonlat instance from the position location.
		// NOTE: The coordinates are transformed to the map's projection by the geolocate control.
		var lonlat = new OpenLayers.LonLat(point.x, point.y);

		//alert("Longitude: " + lonlat.lon + "\nLatitude:    " + lonlat.lat);

		if (!bounds.containsLonLat(lonlat)) {
			alert("Sorry, your current location is outside Netherlands.");
			return;
		}

		// Add the marker
		var marker = new OpenLayers.Marker(lonlat, icon);

		/*
		var popup = null;

		marker.events.on({
			"mouseover": function() {
				if (popup) {
					map.removePopup(popup);
				}
				var content = "<strong>Longitude:</strong> " + lonlat.lon + "<br/>" + "<strong>Latitude:</strong> " + lonlat.lat;
				popup = new OpenLayers.Popup.FramedCloud(
					"popup", lonlat, new OpenLayers.Size(250, 100),
					content,
					null, true, null);
				map.addPopup(popup);
			}
		});
		*/
		markers.addMarker(marker);
	}

	function locateUser() {
		showProgressIndicator();
		geolocate.deactivate();
		geolocate.activate();
	}

	function browserGeoLoocate() {
		// onSuccess Callback
		// This method accepts a Position object, which contains the
		// current GPS coordinates
		//
		var onSuccess = function(position) {
			var message = getPositionMessage(position);
			alert(message);
		};

		// onError Callback receives a PositionError object
		//
		function onError(error) {
			var message = getPositionErrorMessage(error);
			alert(message);
		}

		navigator.geolocation.getCurrentPosition(onSuccess, onError, { enableHighAccuracy: true, timeout: 1000, maximumAge: 1000 });
	}

	/*
	// -------------------------------------------------------------
	//  Basic Navigation
	// -------------------------------------------------------------
	(function() {
		var $frame = $('#basic');
		var $slidee = $frame.children('ul').eq(0);
		var $wrap = $frame.parent();

		// Call Sly on frame
		$frame.sly({
			horizontal: 1,
			itemNav: 'basic',
			smart: 1,
			activateOn: 'click',
			mouseDragging: 1,
			touchDragging: 1,
			releaseSwing: 1,
			startAt: 3,
			scrollBar: $wrap.find('.scrollbar'),
			scrollBy: 1,
			pagesBar: $wrap.find('.pages'),
			activatePageOn: 'click',
			speed: 300,
			elasticBounds: 1,
			easing: 'easeOutExpo',
			dragHandle: 1,
			dynamicHandle: 1,
			clickBar: 1,

			// Buttons
			forward: $wrap.find('.forward'),
			backward: $wrap.find('.backward'),
			prev: $wrap.find('.prev'),
			next: $wrap.find('.next'),
			prevPage: $wrap.find('.prevPage'),
			nextPage: $wrap.find('.nextPage')
		});

		// To Start button
		$wrap.find('.toStart').on('click', function() {
			var item = $(this).data('item');
			// Animate a particular item to the start of the frame.
			// If no item is provided, the whole content will be animated.
			$frame.sly('toStart', item);
		});

		// To Center button
		$wrap.find('.toCenter').on('click', function() {
			var item = $(this).data('item');
			// Animate a particular item to the center of the frame.
			// If no item is provided, the whole content will be animated.
			$frame.sly('toCenter', item);
		});

		// To End button
		$wrap.find('.toEnd').on('click', function() {
			var item = $(this).data('item');
			// Animate a particular item to the end of the frame.
			// If no item is provided, the whole content will be animated.
			$frame.sly('toEnd', item);
		});

		// Add item
		$wrap.find('.add').on('click', function() {
			$frame.sly('add', '<li>' + $slidee.children().length + '</li>');
		});

		// Remove item
		$wrap.find('.remove').on('click', function() {
			$frame.sly('remove', -1);
		});
	}());
	*/

	// -------------------------------------------------------------
	//  Centered Navigation
	// -------------------------------------------------------------
	(function() {
		var listItemsCount = $("#centered ul li").length;
		var $frame = $('#centered');
		var $wrap = $frame.parent();

		// Call Sly on frame
		$frame.sly({
			horizontal: 1,
			itemNav: 'centered',
			smart: 1,
			activateOn: 'click',
			mouseDragging: 1,
			touchDragging: 1,
			releaseSwing: 1,
			//startAt: listItemsCount - 1, //set last item
			startAt: listItemsCount - 2, //set penultimate item
			scrollBar: $wrap.find('.scrollbar'),
			scrollBy: 1,
			speed: 300,
			elasticBounds: 1,
			easing: 'easeOutExpo',
			dragHandle: 1,
			dynamicHandle: 1,
			clickBar: 1,

			// Buttons
			prev: $wrap.find('.prev'),
			next: $wrap.find('.next')
		});
	}());

	/*
	// -------------------------------------------------------------
	//  Force Centered Navigation
	// -------------------------------------------------------------
	(function() {
		var $frame = $('#forcecentered');
		var $wrap = $frame.parent();

		// Call Sly on frame
		$frame.sly({
			horizontal: 1,
			itemNav: 'forceCentered',
			smart: 1,
			activateMiddle: 1,
			activateOn: 'click',
			mouseDragging: 1,
			touchDragging: 1,
			releaseSwing: 1,
			startAt: 0,
			scrollBar: $wrap.find('.scrollbar'),
			scrollBy: 1,
			speed: 300,
			elasticBounds: 1,
			easing: 'easeOutExpo',
			dragHandle: 1,
			dynamicHandle: 1,
			clickBar: 1,

			// Buttons
			prev: $wrap.find('.prev'),
			next: $wrap.find('.next')
		});
	}());
	*/

});